/*
 * Copyright (c) 2009 Lukas Mejdrech
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * - Redistributions of source code must retain the above copyright
 *   notice, this list of conditions and the following disclaimer.
 * - Redistributions in binary form must reproduce the above copyright
 *   notice, this list of conditions and the following disclaimer in the
 *   documentation and/or other materials provided with the distribution.
 * - The name of the author may not be used to endorse or promote products
 *   derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/** @addtogroup net
 *  @{
 */

/** @file
 *  Generic module functions.
 */

#ifndef __NET_MODULES_H__
#define __NET_MODULES_H__
 
#include <async.h>

#include <ipc/ipc.h>
#include <ipc/services.h>

/** Converts the data length between different types.
 *	@param type_from The source type. Input parameter.
 *  @param type_to The destination type. Input parameter.
 *  @param count The number units of the source type size. Input parameter.
 */
#define CONVERT_SIZE( type_from, type_to, count )	(( sizeof( type_from ) / sizeof( type_to )) * ( count ))

/** Registers the module service at the name server.
 *  @param me The module service. Input parameter.
 *  @param phonehash The created phone hash. Output parameter.
 */
#define REGISTER_ME( me, phonehash )	ipc_connect_to_me( PHONE_NS, ( me ), 0, 0, ( phonehash ))

/** Connect to the needed module function type definition.
 *  @param need The needed module service. Input parameter.
 *  @returns The phone of the needed service.
 */
typedef int connect_module_t( services_t need );

/** Connects to the needed module.
 *  @param need The needed module service. Input parameter.
 *  @returns The phone of the needed service.
 */
int connect_to_service( services_t need );

/** Creates bidirectional connection with the needed module service and registers the message receiver.
 *  @param need The needed module service. Input parameter.
 *  @param arg1 The first parameter. Input parameter.
 *  @param arg2 The second parameter. Input parameter.
 *  @param arg3 The third parameter. Input parameter.
 *  @param client_receiver The message receiver. Input parameter.
 *  @returns The phone of the needed service.
 *  @returns Other error codes as defined for the ipc_connect_to_me() function.
 */
int	bind_service( services_t need, ipcarg_t arg1, ipcarg_t arg2, ipcarg_t arg3, async_client_conn_t client_receiver );

#endif

/** @}
 */
